# -*- coding: utf-8 -*-                       
from sys import path
import numpy as np
import odbAccess
import os
from textRepr import *
from timeit import default_timer as timer

def p(pstring):
    prettyPrint(pstring)
    

###################################              
#USER INPUT    
###################################     

# Folder of ODB-file
wdir= 'xxxx'

# # Folder of exported files
wdir_export='xxxx'

# Name of job (odb-file)
JobName='Sulafjorden_TD21_S1-G1'

# Step of modal analysis (default -1 aka last step)
FrequencyStepNumber=-1

# Prefix in front of exported filenames (default empty)
OutputPrefix='Sulafjorden_TD21_S1-G1_'

# Name of assembly (if empty, then automatically select first)
AssemblyName=''

# Export all to txt files rather than npy files (required for ABAQUS2019!)
ExportToTXT=True

###################################              
#START EXPORT BELOW            
###################################     

# OPEN ODB
if JobName[-4:]=='.odb':
    odbJobName=JobName+'.odb'
else:
    odbJobName=JobName+'.odb'
    

myOdb = odbAccess.openOdb(wdir+'\\'+odbJobName)
  
stepNames = myOdb.steps.keys()
nameOfFrequencyStep=stepNames[FrequencyStepNumber]
frequencyStep=myOdb.steps[nameOfFrequencyStep]

# FIND ASSEMBLY NAME IF NOT PROVIDED
if AssemblyName=='' or AssemblyName=='none':
    AssemblyNameKeys=myOdb.steps[nameOfFrequencyStep].historyRegions.keys()
    AssemblyName=AssemblyNameKeys[0]

# NATURAL FREQUENCIES
historyOutputKeys=myOdb.steps[nameOfFrequencyStep].historyRegions[AssemblyName].historyOutputs.keys()

if 'EIGFREQ' in historyOutputKeys:  
    historyOutputFreq=myOdb.steps[nameOfFrequencyStep].historyRegions[AssemblyName].historyOutputs['EIGFREQ']
    if historyOutputFreq.data==None:
        FrequencyVector=np.array([0])
        FrequencyExport=False
        
    else:
        FrequencyVector=[historyOutputFreq.data[j] for j in range(0,len(historyOutputFreq.data))]
        FrequencyVector=np.array(FrequencyVector)[:,1]
        FrequencyExport=True
        
    
else:
    FrequencyVector=np.array([0])
    FrequencyExport=False
    

if 'DAMPRATIO' in historyOutputKeys:                   
    historyOutputDamp=myOdb.steps[nameOfFrequencyStep].historyRegions[AssemblyName].historyOutputs['DAMPRATIO']
    DampingVector=[historyOutputDamp.data[j] for j in range(0,len(historyOutputDamp.data))]
    DampingVector=np.array(DampingVector)[:,1]
    DampingExport=True
    
else:
    DampingVector=np.array([0])
    DampingExport=False
    

if 'EIGREAL' in historyOutputKeys:
    historyOutputEigreal=myOdb.steps[nameOfFrequencyStep].historyRegions[AssemblyName].historyOutputs['EIGREAL']
    EigrealVector=[historyOutputEigreal.data[j] for j in range(0,len(historyOutputEigreal.data))]
    EigrealVector=np.array(EigrealVector)[:,1]
    EigrealExport=True
else:
    EigrealVector=np.array([0])
    EigrealExport=False


if 'EIGIMAG' in historyOutputKeys:                  
    historyOutputEigimag=myOdb.steps[nameOfFrequencyStep].historyRegions[AssemblyName].historyOutputs['EIGIMAG']
    EigimagVector=[historyOutputEigimag.data[j] for j in range(0,len(historyOutputEigimag.data))]
    EigimagVector=np.array(EigimagVector)[:,1]
    EigimagExport=True
else:
    EigimagVector=np.array([0])              
    EigimagExport=False
    

if 'GM' in historyOutputKeys:              
    historyOutputGenmass=myOdb.steps[nameOfFrequencyStep].historyRegions[AssemblyName].historyOutputs['GM']
    GenmassVector=[historyOutputGenmass.data[j] for j in range(0,len(historyOutputGenmass.data))]
    GenmassVector=np.array(GenmassVector)[:,1]
    GenmassExport=True
else:
    GenmassVector=np.array([0])
    GenmassExport=False
    

###################################              
#MODE SHAPES              
###################################              
t_start=timer()

PhiExport=True

#find number of modes and nodes              
numZ=len(frequencyStep.frames)
frequencyFrame=frequencyStep.frames[1]
numN=len(frequencyFrame.fieldOutputs['U'].values)          

ModeshapeMatrix=np.zeros((numN*6,numZ))
ModeshapeConjMatrix=np.zeros((numN*6,numZ))

PhiLabelVector=[]; 
for z in range(len(frequencyStep.frames)): #for each mode
    
    frequencyFrame=frequencyStep.frames[z]
    
    modeTrans=frequencyFrame.fieldOutputs['U']
    IndexU=np.array([0, 1, 2])
    
    modeRot=frequencyFrame.fieldOutputs['UR']
    IndexUR=np.array([0, 1, 2])+np.array([3, 3, 3])
    
    # Check if conjugate data is relevant (imaginary mode, rarely used)
    if modeTrans.values[0].conjugateData==None:
        PhiConjExport=False
    else:
        PhiConjExport=True

    for n in range(len(frequencyFrame.fieldOutputs['U'].values)): #for each node
    
        phiZ_nodeN_U=modeTrans.values[n].data
        ModeshapeMatrix[IndexU,z]=phiZ_nodeN_U
        
        phiZ_nodeN_UR=modeRot.values[n].data
        ModeshapeMatrix[IndexUR,z]=phiZ_nodeN_UR
                
        if PhiConjExport==True:
        
            phiZ_nodeN_U_conj=modeTrans.values[n].conjugateData
            ModeshapeConjMatrix[IndexU,z]=phiZ_nodeN_U_conj
            
            phiZ_nodeN_UR_conj=modeRot.values[n].conjugateData
            ModeshapeConjMatrix[IndexUR,z]=phiZ_nodeN_UR_conj
        
        IndexU=IndexU+6
        IndexUR=IndexUR+6   
        
        if z==0: 
            PhiLabelVectorN=[str(modeTrans.values[n].nodeLabel) + '_' + s  for s in modeTrans.componentLabels] + [str(modeRot.values[n].nodeLabel) + '_' + s  for s in modeRot.componentLabels]
            PhiLabelVector.extend(PhiLabelVectorN)
        

ModeshapeMatrix=ModeshapeMatrix[:,1:] #cut zero mode
ModeshapeConjMatrix=ModeshapeConjMatrix[:,1:] #cut zero mode 
PhiLabelVector=np.array(PhiLabelVector).flatten()

t_end=timer()
print('Time #MODE SHAPES ' + str(t_end-t_start))

###################################
#MODE SECTION FORCES
###################################
t_start=timer()

#find number of modes and nodes

frequencyFrame=frequencyStep.frames[0]
PhiSFExport=False
if 'SF' in frequencyFrame.fieldOutputs.keys():
    PhiSFExport=True
    

ModeSectionForceMatrix=np.array([0 , 0 , 0])
ElementLabelVector=np.array(['none'])

if PhiSFExport==True:
    numZ=len(frequencyStep.frames)
    frequencyFrame=frequencyStep.frames[1]
    numN=len(frequencyFrame.fieldOutputs['SF'].values)
    
    ModeSectionForceMatrix=np.zeros((numN*6,numZ))
    ElementLabelVector=[]
    for z in range(len(frequencyStep.frames)): #for each mode

        frequencyFrame=frequencyStep.frames[z]
        modeSF=frequencyFrame.fieldOutputs['SF']
        modeSM=frequencyFrame.fieldOutputs['SM']
        
        IndexA=np.array([0, 1, 2])
        
        for n in range(len(frequencyFrame.fieldOutputs['SF'].values)): #for each node
            
            if (modeSF.values[n].baseElementType=='B31')==False and (modeSF.values[n].baseElementType=='B32')==False:
                continue
            
            phiZ_nodeN_SF=modeSF.values[n].data
            ModeSectionForceMatrix[IndexA,z]=phiZ_nodeN_SF
            IndexA=IndexA+3
            
            phiZ_nodeN_SM=modeSM.values[n].data
            ModeSectionForceMatrix[IndexA,z]=phiZ_nodeN_SM
            IndexA=IndexA+3
            
            if z==0:
                modeSM_componentLabels_replace=('SM1', 'SM2', 'SM3') # error in abaqus documentation? States 2 1 3
                ElementLabelVectorN=[str(modeSF.values[n].elementLabel) + '_' + s  for s in modeSF.componentLabels] + [str(modeSM.values[n].elementLabel) + '_' + s  for s in modeSM_componentLabels_replace]
                ElementLabelVector.extend(ElementLabelVectorN)
    
    #IndexA=IndexA-3
    ModeSectionForceMatrix=ModeSectionForceMatrix[0:IndexA[0]:1,1:] #cut zero mode
    ElementLabelVector=np.array(ElementLabelVector).flatten()

t_end=timer()
print('Time #MODAL SECTION FORCES ' + str(t_end-t_start))

###################################
#NODE COORDINATES. EXTRACTION DONE AT CONFIG WHERE FREQUENCIES ARE CALCULATED
################################### 
t_start=timer()

initialFrame = frequencyStep.frames[0]
NodeCoordExport=False
if 'COORD' in initialFrame.fieldOutputs.keys():    
    NodeCoordExport=True

NodeCoordMatrix=np.array([0 , 0 , 0])
NodeCoordLabelVector=np.array([0 ])
if NodeCoordExport==True:
    coordValues=initialFrame.fieldOutputs['COORD'].values
    
    NodeCoordMatrix=[coordValues[ii].data for ii in range(len(coordValues))]
    NodeCoordLabelVector=[coordValues[ii].nodeLabel for ii in range(len(coordValues))]
    NodeCoordMatrix=np.array(NodeCoordMatrix)


t_end=timer()
print('Time NODE COORDINATES ' + str(t_end-t_start))

###################################
# SECTION FORCES AT DEAD LOAD. EXTRACTION DONE AT CONFIG WHERE FREQUENCIES ARE CALCULATED
###################################
t_start=timer()

nameOfTensionStep=stepNames[FrequencyStepNumber]
tensionStep=myOdb.steps[nameOfTensionStep]    
tensionFrame = tensionStep.frames[0] 

SF_Matrix=np.array([0 , 0 , 0])
SM_Matrix=np.array([0 , 0 , 0])
ElementLabelVector_SF=np.array([0])
ElementLabelVector_SM=np.array([0])

if 'SF' in tensionFrame.fieldOutputs.keys():       
    
    SFExport=True
    
    SF_SectionForces=tensionFrame.fieldOutputs['SF'].values
    elementLabelAll=[ SF_SectionForces[ii].elementLabel  for ii in range(len(SF_SectionForces))]
    # elementLabelAll=np.unique(elementLabelAll)
    
    # indexElementForceExport=[elementLabelAll.index(elementLabelForceExport[ii]) for ii in range(len(elementLabelAll))]
    indexElementForceExport=range(len(elementLabelAll))
    SF_Matrix=[SF_SectionForces[ii].data[0:3]  for ii in indexElementForceExport]
    SF_Matrix=np.array(SF_Matrix)
    
    SM_SectionForces=tensionFrame.fieldOutputs['SM'].values
    SM_Matrix=[SM_SectionForces[ii].data[0:3]  for ii in indexElementForceExport]
    SM_Matrix=np.array(SM_Matrix)
    
    # ELEMENTLABEL
    ElementLabelVector_SF=[ SF_SectionForces[ii].elementLabel  for ii in indexElementForceExport]
    ElementLabelVector_SF=np.array(ElementLabelVector_SF)
    
    ElementLabelVector_SM=[ SM_SectionForces[ii].elementLabel  for ii in indexElementForceExport]
    ElementLabelVector_SM=np.array(ElementLabelVector_SM).flatten()
    
    # PhiLabelVector=[ [ [str(SF_SectionForces[ii].elementLabel) + '_' + s  for s in fieldOutputSF.componentLabels] , [str(modeRot.values[ii].nodeLabel) + '_' + s  for s in modeRot.componentLabels] ] for ii in indexElementForceExport]
    # PhiLabelVector=np.array(PhiLabelVector).flatten()
    

t_end=timer()
print('Time #SF AT DEAD LOAD ' + str(t_end-t_start))

###################################              
# ELEMENTS CONNECTIVITY
###################################  
t_start=timer()

ElementConnectivityExport=True

ElementConnectivity2NodeMatrix=np.array([ [0 , 0 , 0 ],[0 , 0 , 0 ]])
ElementConnectivity3NodeMatrix=np.array([ [0 , 0 , 0 , 0],[0 , 0 , 0 , 0]])

InstanceKeys=myOdb.rootAssembly.instances.keys()

for k in range(len(InstanceKeys)):

    myInstance=myOdb.rootAssembly.instances[InstanceKeys[k]]
    myInstanceElements=myInstance.elements
    
    j_select=[]
    for j in range(len(myInstanceElements)):
        if myInstanceElements[j].type=='B31' or myInstanceElements[j].type=='B33':
            j_select.append(j)
    
    ElementConnectivity2NodeMatrix_temp=np.array( [ [myInstanceElements[j].label , int(myInstanceElements[j].connectivity[0]) , int(myInstanceElements[j].connectivity[1]) ]  for j in j_select ] )
    
    if len(ElementConnectivity2NodeMatrix_temp)>0:    
        ElementConnectivity2NodeMatrix=np.vstack((ElementConnectivity2NodeMatrix,ElementConnectivity2NodeMatrix_temp))
    
    j_select=[]
    for j in range(len(myInstanceElements)):
        if myInstanceElements[j].type=='B32':
            j_select.append(j)
    
    ElementConnectivity3NodeMatrix_temp=np.array( [ [myInstanceElements[j].label , int(myInstanceElements[j].connectivity[0]) , int(myInstanceElements[j].connectivity[1]) , int(myInstanceElements[j].connectivity[2]) ]  for j in j_select ] )
    
    if len(ElementConnectivity3NodeMatrix_temp)>0:    
        ElementConnectivity3NodeMatrix=np.vstack((ElementConnectivity3NodeMatrix,ElementConnectivity3NodeMatrix_temp))


ElementConnectivity2NodeMatrix=ElementConnectivity2NodeMatrix[2:,:] #cut dummy values 
ElementConnectivity3NodeMatrix=ElementConnectivity3NodeMatrix[2:,:] #cut dummy values 

t_end=timer()
print('Time #ELEMENTS ' + str(t_end-t_start))


###################################
#EXPORT
###################################


def FunctionSaveToTXT(dir_save,name_save,OutputPrefix,atype,A_matrix):

    # Inputs:
    # dir_save: string 
    # name_save: string 
    # OutputPrefix: string 
    # atype: string 
    # A_matrix: numpy array
    
    if atype=='number':
        np.savetxt((dir_save+'\\'+OutputPrefix+name_save+'.txt'), A_matrix , delimiter = ',') 
    elif atype=='string':
        np.savetxt((dir_save+'\\'+OutputPrefix+name_save+'.txt'), A_matrix , delimiter = ' ', fmt='%s') 


# OutputPrefix=''
t_start=timer()

# ADD PREFIX
if OutputPrefix=='none':
    OutputPrefix=''
    

if FrequencyExport:
    if ExportToTXT==True:
        FunctionSaveToTXT(wdir_export,'freq',OutputPrefix,'number',FrequencyVector)
    else:
        np.save((wdir_export+outputPrefix+'freq'), FrequencyVector)
    

if DampingExport:
    if ExportToTXT==True:
        FunctionSaveToTXT(wdir_export,'damp',OutputPrefix,'number',DampingVector)
    else:
        np.save((wdir_export+outputPrefix+'damp'), DampingVector)
    

if EigrealExport:
    if ExportToTXT==True:
        FunctionSaveToTXT(wdir_export,'eigreal',OutputPrefix,'number',EigrealVector)
    else:
        np.save((wdir_export+outputPrefix+'eigreal'), EigrealVector)


if EigimagExport:
    if ExportToTXT==True:
        FunctionSaveToTXT(wdir_export,'eigimag',OutputPrefix,'number',EigimagVector)
    else:    
        np.save((wdir_export+outputPrefix+'eigimag'), EigimagVector)


if GenmassExport:
    if ExportToTXT==True:
        FunctionSaveToTXT(wdir_export,'genmass',OutputPrefix,'number',GenmassVector)
    else:
        np.save((wdir_export+outputPrefix+'genmass'), GenmassVector)
    

if PhiExport:
    if ExportToTXT==True:
        FunctionSaveToTXT(wdir_export,'phi',OutputPrefix,'number',ModeshapeMatrix)
    else:    
        np.save((wdir_export+outputPrefix+'phi'), ModeshapeMatrix)
    
    FunctionSaveToTXT(wdir_export,'phi_label',OutputPrefix,'string',PhiLabelVector)


if PhiConjExport:
    if ExportToTXT==True:
        FunctionSaveToTXT(wdir_export,'phi_conj',OutputPrefix,'number',ModeshapeConjMatrix)
    else:    
        np.save((wdir_export+outputPrefix+'phi_conj'), ModeshapeConjMatrix)


if PhiSFExport:
    if ExportToTXT==True:
        FunctionSaveToTXT(wdir_export,'phi_sf',OutputPrefix,'number',ModeSectionForceMatrix)
    else:    
        np.save((wdir_export+outputPrefix+'phi_sf'), ModeSectionForceMatrix)
    
    FunctionSaveToTXT(wdir_export,'phi_sf_label',OutputPrefix,'string',ElementLabelVector)


if NodeCoordExport:
    if ExportToTXT==True:
        FunctionSaveToTXT(wdir_export,'nodecoord',OutputPrefix,'number',NodeCoordMatrix)
        FunctionSaveToTXT(wdir_export,'nodecoord_label',OutputPrefix,'number',NodeCoordLabelVector)
    else:    
        np.save((wdir_export+outputPrefix+'nodecoord'), NodeCoordMatrix)
        np.save((wdir_export+outputPrefix+'nodecoord_label'), NodeCoordLabelVector)
        


if SFExport:
    if ExportToTXT==True:
        FunctionSaveToTXT(wdir_export,'SF',OutputPrefix,'number',SF_Matrix)
        FunctionSaveToTXT(wdir_export,'SM',OutputPrefix,'number',SM_Matrix)
    else:
        np.save((wdir_export+outputPrefix+'SF'), SF_Matrix)
        np.save((wdir_export+outputPrefix+'SM'), SM_Matrix)
    
    FunctionSaveToTXT(wdir_export,'SF_label',OutputPrefix,'string',ElementLabelVector_SF)
    FunctionSaveToTXT(wdir_export,'SM_label',OutputPrefix,'string',ElementLabelVector_SM)

if ElementConnectivityExport:
    if ExportToTXT==True:
        FunctionSaveToTXT(wdir_export,'ElementConnectivity2Node',OutputPrefix,'number',ElementConnectivity2NodeMatrix)
        FunctionSaveToTXT(wdir_export,'ElementConnectivity3Node',OutputPrefix,'number',ElementConnectivity3NodeMatrix)
    else:   
        np.save((wdir_export+outputPrefix+'ElementConnectivity2Node'), ElementConnectivity2NodeMatrix)
        np.save((wdir_export+outputPrefix+'ElementConnectivity3Node'), ElementConnectivity3NodeMatrix)
    



t_end=timer()
print('Time #EXPORT ' + str(t_end-t_start))


###################################
#CLOSE
###################################

#myOdb.close()
    
# sys.exit(0)


################################################################
################################################################
################################################################
################################################################
################################################################
################################################################
