clc
clear all 
close all

% CALCULATIONS OF SCRUTON NUMBER
% Sc=(m*zeta)/(rho*D^2)

% INPUT:
% m:   	mass per unit length [kgm^-1]
% zeta: damping ratio
% rho:  density [kgm^-3]
% D:    characteristic dimension [m]

%% SECTION MODELS
D_m=0.05;
G_m=[0.11 0.21 0.31];
rho_m=1.2;
m_m=[2.1522 2.1522 2.1522;
    2.1278 2.1278 2.1278;
    2.1998 2.1998 2.1998];
zeta_m=0.004;
Sc_m=zeros(3);

for i=1:3 % rows=section
    for j=1:3 % columns=gaps
        Sc_m(i,j)=m_m(i,j)*zeta_m/(rho_m*D_m^2);
    end
end
Sc_m

fig1=figure; hold on; grid on; grid minor; box on; 
plot(G_m/D_m,Sc_m(1,:),'--bo', 'MarkerSize',14,'LineWidth',1.2)
plot(G_m/D_m,Sc_m(2,:),'--ro', 'MarkerSize',14,'LineWidth',1.2)
plot(G_m/D_m,Sc_m(3,:),'--o','color',[0 0.6 0.3], 'MarkerSize',14,'LineWidth',1.2)
xlabel('$\mathrm{gap/depth\:[-]}$','Interpreter','Latex')
ylabel('$\mathrm{Sc\:[-]}$','Interpreter','Latex')
xlim([1.4 7])
ylim([2.5 3.5])
legend('S1','S2','S3')
ax = gca; ax.FontSize = 16;
xticks([2 4 6])
yticks([2.6 2.8 3 3.2 3.4])


saveas(fig1,'Sc_models.jpg')

%% FULL-SCALE BRIDGE
% Rows=section, columns=gaps
D_b=2.5;
rho_b=1.25;
G_b=[5.5 10.5 15.5];
m_b=[19264 19612 19959; 19385 19732 20080; 19760 20108 20455];

%m_b=[29498 29846 30193; 29619 29966 30314; 29995 30342 30690]

zeta_b=0.002;
Sc_b=zeros(3);

for i=1:3 % rows=section
    for j=1:3 % columns=gaps
        Sc_b(i,j)=m_b(i,j)*zeta_b/(rho_b*D_b^2);
    end
end
Sc_b

fig2=figure; hold on; grid on; grid minor; box on; 
plot(G_b/D_b,Sc_b(1,:),'--bo', 'MarkerSize',14,'LineWidth',1.2)
plot(G_b/D_b,Sc_b(2,:),'--ro', 'MarkerSize',14,'LineWidth',1.2)
plot(G_b/D_b,Sc_b(3,:),'--o','color',[0 0.6 0.3], 'MarkerSize',14,'LineWidth',1.2)
xlabel('$\mathrm{gap/depth\:[-]}$','Interpreter','Latex')
ylabel('$\mathrm{Sc\:[-]}$','Interpreter','Latex')

ylim([4.7 5.7])
xlim([1.4 7])
legend('S1','S2','S3')
ax = gca; ax.FontSize = 16;
xticks([2 4 6])

saveas(fig2,'Sc_full-scale.jpg')



